{
ANSI =>
	{
	magic=> 'bright_red on_bright_yellow',
	
	reset => 'reset',
	
	white => 'white',
	black => 'black',
	green => 'green',
	yellow => 'yellow',
	cyan => 'cyan',
	red => 'red',
	blue => 'blue',
	magenta => 'magenta',
	
	bright_white => 'bright_white',
	bright_black => 'bright_black',
	bright_green => 'bright_green',
	bright_yellow => 'bright_yellow',
	bright_cyan => 'bright_cyan',
	bright_red => 'bright_red',
	bright_blue => 'bright_blue',
	bright_magenta => 'bright_magenta',
	
	on_white => 'on_white',
	on_black => 'on_black',
	on_green => 'on_green',
	on_yellow => 'on_yellow',
	on_cyan => 'on_cyan',
	on_red => 'on_red',
	on_blue => 'on_blue',
	on_magenta => 'on_magenta',
	
	on_bright_white => 'on_bright_white',
	on_bright_black => 'on_bright_black',
	on_bright_green => 'on_bright_green',
	on_bright_yellow => 'on_bright_yellow',
	on_bright_cyan => 'on_bright_cyan',
	on_bright_red => 'on_bright_red',
	on_bright_blue => 'on_bright_blue',
	on_bright_magenta => 'on_bright_magenta',
	},
	
HTML =>
	{
	magic => 'color:#f00; background-color:#ff0; ',
	
	# any attribute you can put in a span
	reset => '',
	
	white => 'color:#aaa; ',
	black => 'color:#000; ',
	green => 'color:#0a0; ',
	yellow => 'color:#aa0; ',
	cyan => 'color:#0aa; ',
	red => 'color:#a00; ',
	blue => 'color:#00a; ',
	magenta => 'color:#a0a; ',
	
	bright_white => 'color:#fff; ',
	bright_black => 'color:#000; ',
	bright_green => 'color:#0f0; ',
	bright_yellow => 'color:#ff0; ',
	bright_cyan => 'color:#0ff; ',
	bright_red => 'color:#f00; ',
	bright_blue => 'color:#00f; ',
	bright_magenta => 'color:#f0f; ',

	on_white => 'background-color:#aaa; ',
	on_black => 'background-color:#000; ',
	on_green => 'background-color:#0a0; ',
	on_yellow => 'background-color:#aa0; ',
	on_cyan => 'background-color:#0aa; ',
	on_red => 'background-color:#a00; ',
	on_blue => 'background-color:#00a; ',
	on_magenta => 'background-color:#a0a; ',
	
	on_bright_white => 'background-color:#fff; ',
	on_bright_black => 'background-color:#000; ',
	on_bright_green => 'background-color:#0f0; ',
	on_bright_yellow => 'background-color:#ff0; ',
	on_bright_cyan => 'background-color:#0ff; ',
	on_bright_red => 'background-color:#f00; ',
	on_bright_blue => 'background-color:#00f; ',
	on_bright_magenta => 'background-color:#f0f; ',
	},
},